/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/EvolverFloatVector.cpp
 *  \brief  Source code of class GA::EvolverFloatVector.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.8 $
 *  $Date: 2004/04/25 23:12:19 $
 */

#include "beagle/GA.hpp"

#include <string>

using namespace Beagle;


/*!
 *  \brief Construct a real-valued GA evolver.
 *  \param inInitSize Size of the GA float vectors.
 */
GA::EvolverFloatVector::EvolverFloatVector(IntegerVector inInitSize)
{
  addOperator(new GA::InitFltVecOp(inInitSize));
  addOperator(new GA::CrossoverBlendFltVecOp);
  addOperator(new GA::CrossoverOnePointFltVecOp);
  addOperator(new GA::CrossoverTwoPointsFltVecOp);
  addOperator(new GA::CrossoverUniformFltVecOp);
  addOperator(new GA::MutationGaussianFltVecOp);
}


/*!
 *  \brief Construct a real-valued GA evolver.
 *  \param inEvalOp Evaluation operator.
 *  \param inInitSize Size of the GA float vectors.
 */
GA::EvolverFloatVector::EvolverFloatVector(EvaluationOp::Handle inEvalOp, IntegerVector inInitSize)
{
  addOperator(inEvalOp);
  addOperator(new GA::InitFltVecOp(inInitSize));
  addOperator(new GA::CrossoverBlendFltVecOp);
  addOperator(new GA::CrossoverOnePointFltVecOp);
  addOperator(new GA::CrossoverTwoPointsFltVecOp);
  addOperator(new GA::CrossoverUniformFltVecOp);
  addOperator(new GA::MutationGaussianFltVecOp);

  addBootStrapOp("IfThenElseOp");
  IfThenElseOp::Handle lITE = castHandleT<IfThenElseOp>(getBootStrapSet().back());
  lITE->setConditionTag("ms.restart.file");
  lITE->setConditionValue("");
  lITE->insertPositiveOp("GA-InitFltVecOp", getOperatorMap());
  lITE->insertPositiveOp(inEvalOp->getName(), getOperatorMap());
  lITE->insertPositiveOp("StatsCalcFitnessSimpleOp", getOperatorMap());
  lITE->insertNegativeOp("MilestoneReadOp", getOperatorMap());
  addBootStrapOp("TermMaxGenOp");
  addBootStrapOp("MilestoneWriteOp");

  addMainLoopOp("SelectTournamentOp");
  addMainLoopOp("GA-CrossoverBlendFltVecOp");
  addMainLoopOp("GA-MutationGaussianFltVecOp");
  addMainLoopOp(inEvalOp->getName());
  addMainLoopOp("MigrationRandomRingOp");
  addMainLoopOp("StatsCalcFitnessSimpleOp");
  addMainLoopOp("TermMaxGenOp");
  addMainLoopOp("MilestoneWriteOp");
}


